import mock
from testtools import TestCase
# import unittest
from charmhelpers.contrib.benchmark import Benchmark, action_set  # noqa
from tests.helpers import patch_open, FakeRelation

TO_PATCH = [
    'in_relation_hook',
    'relation_ids',
    'relation_set',
    'relation_get',
]

FAKE_RELATION = {
    'benchmark:0': {
        'benchmark/0': {
            'hostname': '127.0.0.1',
            'port': '1111',
            'graphite_port': '2222',
            'graphite_endpoint': 'http://localhost:3333',
            'api_port': '4444'
        }
    }
}


class TestBenchmark(TestCase):

    def setUp(self):
        super(TestBenchmark, self).setUp()
        for m in TO_PATCH:
            setattr(self, m, self._patch(m))
        self.fake_relation = FakeRelation(FAKE_RELATION)
        # self.hook_name.return_value = 'benchmark-relation-changed'

        self.relation_get.side_effect = self.fake_relation.get
        self.relation_ids.side_effect = self.fake_relation.relation_ids

    def _patch(self, method):
        _m = mock.patch('charmhelpers.contrib.benchmark.' + method)
        m = _m.start()
        self.addCleanup(_m.stop)
        return m

    @mock.patch('os.path.exists')
    @mock.patch('subprocess.check_output')
    def test_benchmark_start(self, check_output, exists):

        exists.return_value = True
        check_output.return_value = "data"

        with patch_open() as (_open, _file):
            self.assertIsNone(Benchmark().start())
            # _open.assert_called_with('/etc/benchmark.conf', 'w')

        COLLECT_PROFILE_DATA = '/usr/local/bin/collect-profile-data'
        exists.assert_any_call(COLLECT_PROFILE_DATA)
        check_output.assert_any_call([COLLECT_PROFILE_DATA])

    def test_benchmark_finish(self):
        with patch_open() as (_open, _file):
            self.assertIsNone(Benchmark().finish())
            # _open.assert_called_with('/etc/benchmark.conf', 'w')

    @mock.patch('charmhelpers.contrib.benchmark.find_executable')
    @mock.patch('charmhelpers.contrib.benchmark.subprocess.check_call')
    def test_benchmark_action_set(self, check_call, find_executable):
        find_executable.return_value = "/usr/bin/action-set"
        self.assertTrue(action_set('foo', 'bar'))

        find_executable.assert_called_once_with('action-set')
        check_call.assert_called_once_with(['action-set', 'foo=bar'])

    @mock.patch('charmhelpers.contrib.benchmark.find_executable')
    @mock.patch('charmhelpers.contrib.benchmark.subprocess.check_call')
    def test_benchmark_action_set_dict(self, check_call, find_executable):
        find_executable.return_value = "/usr/bin/action-set"
        self.assertTrue(action_set('baz', {'foo': 1, 'bar': 2}))

        find_executable.assert_called_with('action-set')

        check_call.assert_any_call(['action-set', 'baz.foo=1'])
        check_call.assert_any_call(['action-set', 'baz.bar=2'])

    @mock.patch('charmhelpers.contrib.benchmark.relation_get')
    @mock.patch('charmhelpers.contrib.benchmark.relation_set')
    @mock.patch('charmhelpers.contrib.benchmark.relation_ids')
    @mock.patch('charmhelpers.contrib.benchmark.in_relation_hook')
    def test_benchmark_init(self, in_relation_hook, relation_ids, relation_set, relation_get):

        in_relation_hook.return_value = True
        relation_ids.return_value = ['benchmark:0']
        actions = ['asdf', 'foobar']

        with patch_open() as (_open, _file):
            b = Benchmark(actions)

            self.assertIsInstance(b, Benchmark)

            self.assertTrue(relation_get.called)
            self.assertTrue(relation_set.called)

            relation_ids.assert_called_once_with('benchmark')

            for key in b.required_keys:
                relation_get.assert_any_call(key)

            relation_set.assert_called_once_with(
                relation_id='benchmark:0',
                relation_settings={'benchmarks': ",".join(actions)}
            )

            _open.assert_called_with('/etc/benchmark.conf', 'w')
            for key, val in iter(FAKE_RELATION['benchmark:0']['benchmark/0'].items()):
                _file.write.assert_any_called("%s=%s\n" % (key, val))
